Package: TSCAN
Type: Package
Title: Tools for Single-Cell Analysis
Version: 1.50.0
Date: 2024-08-14
Authors@R: c(
        person("Zhicheng", "Ji", role=c("aut", "cre"), email="zji4@jhu.edu"),
        person("Hongkai", "Ji", role=c("aut")),
        person("Aaron", "Lun", role=c("ctb"), email="infinite.monkeys.with.keyboards@gmail.com")
    )
Description: Provides methods to perform trajectory analysis based on a
        minimum spanning tree constructed from cluster centroids.
        Computes pseudotemporal cell orderings by mapping cells in each
        cluster (or new cells) to the closest edge in the tree. Uses
        linear modelling to identify differentially expressed genes
        along each path through the tree. Several plotting and
        interactive visualization functions are also implemented.
License: GPL(>=2)
Depends: R (>= 4.4.0), SingleCellExperiment, TrajectoryUtils
Imports: ggplot2, shiny, plyr, grid, fastICA, igraph, combinat, mgcv,
        mclust, gplots, methods, stats, Matrix, SummarizedExperiment,
        SparseArray (>= 1.5.23), DelayedArray (>= 0.31.9), S4Vectors
VignetteBuilder: knitr
Suggests: knitr, testthat, scuttle, scran, metapod, BiocParallel,
        BiocNeighbors, batchelor
biocViews: GeneExpression, Visualization, GUI
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/TSCAN
git_branch: RELEASE_3_15
git_last_commit: f9c8b02
git_last_commit_date: 2022-04-26
NeedsCompilation: no
Packaged: 2026-04-29 15:06:35 UTC; root
Author: Zhicheng Ji [aut, cre], Hongkai Ji [aut], Aaron Lun [ctb]
Maintainer: Zhicheng Ji <zji4@jhu.edu>
Config/pak/sysreqs: cmake libglpk-dev make libuv1-dev libxml2-dev
        zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:39:43 UTC
RemoteUrl: https://github.com/bioc/TSCAN
RemoteRef: RELEASE_3_23
RemoteSha: f7afcb41e89c8011585a75114446c0da75e2f45b
Built: R 4.6.0; ; 2026-04-29 15:50:16 UTC; unix
