# Generated by roxygen2: do not edit by hand

export(TSCANorder)
export(TSCANui)
export(createClusterMST)
export(difftest)
export(exprmclust)
export(mapCellsToEdges)
export(orderCells)
export(orderscore)
export(perCellEntropy)
export(plotmclust)
export(preprocess)
export(quickPseudotime)
export(reportEdges)
export(rowmean)
export(singlegeneplot)
export(testPseudotime)
exportMethods(mapCellsToEdges)
exportMethods(perCellEntropy)
exportMethods(quickPseudotime)
exportMethods(reportEdges)
exportMethods(testPseudotime)
import(combinat)
import(fastICA)
import(ggplot2)
import(gplots)
import(grid)
import(igraph)
import(methods)
import(mgcv)
import(plyr)
import(shiny)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importClassesFrom(SparseArray,COO_SparseMatrix)
importClassesFrom(SparseArray,SVT_SparseMatrix)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(DelayedArray,blockApply)
importFrom(DelayedArray,colAutoGrid)
importFrom(Matrix,colSums)
importFrom(Matrix,t)
importFrom(Matrix,which)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,List)
importFrom(S4Vectors,metadata)
importFrom(SingleCellExperiment,colLabels)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDims)
importFrom(SparseArray,nzwhich)
importFrom(SummarizedExperiment,assay)
importFrom(TrajectoryUtils,PseudotimeOrdering)
importFrom(TrajectoryUtils,createClusterMST)
importFrom(TrajectoryUtils,pathStat)
importFrom(TrajectoryUtils,rowmean)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,adjacent_vertices)
importFrom(igraph,components)
importFrom(igraph,degree)
importFrom(igraph,get.edge.ids)
importFrom(mclust,Mclust)
importFrom(mclust,mclustBIC)
importFrom(stats,aggregate)
importFrom(stats,cutree)
importFrom(stats,fitted.values)
importFrom(stats,hclust)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,sd)
