#' TSCAN: Tools for Single-Cell ANalysis
#'
#' This package provides essential tools used in analyzing data from single-cell experiments
#'
#' TSCAN enables users to easily construct and tune pseudotemporal cell ordering as well as analyzing differentially expressed genes. 
#' TSCAN comes with a user-friendly GUI written in shiny. More functions will come in the future.
#' 
#' @docType package
#' @import methods
#' @name TSCAN
#' @importFrom stats aggregate cutree fitted.values hclust
#' lm p.adjust pchisq prcomp predict sd
NULL
